@@ -23,8 +23,7 @@ extern int desktop_use_shaped_icons;	/* Whether to use shaped icons or not (for
 extern int desktop_auto_placement;	/* Whether to auto-place icons or not (user placement) */
 extern int desktop_snap_icons;		/* Whether to snap icons to the grid or not */
 
-extern int want_transparent_icons;
-extern int want_transparent_text;
+extern int tree_panel_visible;	        
 
 /* Initializes the desktop -- init DnD, load the default desktop icons, etc. */
 void desktop_init (void);
@@ -43,9 +42,6 @@ typedef struct {
 					 */
 	int selected : 1;		/* Is the icon selected? */
 	int tmp_selected : 1;		/* Temp storage for original selection while rubberbanding */
-	int finishing_selection : 1;	/* Flag set while we are releasing
-					 * button after selecting in the text
-					 */
 } DesktopIconInfo;
 
 void desktop_icon_info_destroy   (DesktopIconInfo *dii);
@@ -68,5 +64,7 @@ typedef struct {
 	char              *pathname;
 } desktop_icon_t;
 
+file_entry *file_entry_from_file (char *filename);
+void        file_entry_free      (file_entry *fe);
 
 #endif
