@@ -9,6 +9,7 @@
 #include "x.h"
 #include <stdio.h>
 #include <sys/stat.h>
+#include <unistd.h>
 #include "dir.h"
 #include "panel.h"
 #include "gscreen.h"
@@ -24,6 +25,9 @@
 #include "setup.h"
 #include "../vfs/vfs.h"
 #include "gprefs.h"
+
+
+
 #define UNDEFINED_INDEX -1
 
 GList *containers = 0;
@@ -275,9 +279,16 @@ run_cmd (void)
 void configure_box (void);
 
 GtkCheckMenuItem *gnome_toggle_snap (void);
+GnomeUIInfo gnome_panel_new_menu [] = {
+	{ GNOME_APP_UI_ITEM, N_("_Terminal"), N_("Launch a new terminal in the current directory"), gnome_open_terminal},
+	/* If this ever changes, make sure you update create_new_menu accordingly. */
+	{ GNOME_APP_UI_ITEM, N_("_Directory..."), N_("Creates a new directory"), gnome_mkdir_cmd },
+	{ GNOME_APP_UI_ENDOFINFO, 0, 0 }
+};
+
 
 GnomeUIInfo gnome_panel_file_menu [] = {
-	GNOMEUIINFO_MENU_NEW_ITEM(N_("_New window"), N_("Opens a new window"), gnome_open_panel, NULL),
+	GNOMEUIINFO_MENU_NEW_ITEM(N_("New _Window"), N_("Opens a new window"), gnome_open_panel, NULL),
 	
 	/* We want to make a new menu entry here... */
 	/* For example: */
@@ -287,14 +298,13 @@ GnomeUIInfo gnome_panel_file_menu [] = {
 	/*  Gnumeric Spreadsheet */
 	/*  Text Document */
 	/*  etc... */
-
+	{ GNOME_APP_UI_SUBTREE, N_("_New"), 	       NULL, gnome_panel_new_menu},
 	{ GNOME_APP_UI_SEPARATOR },
 	{ GNOME_APP_UI_ITEM, N_("_Open"),              N_("Opens the selected files"), gnome_open_files },
 /*	{ GNOME_APP_UI_ITEM, N_("Open _FTP site"),     N_("Opens an FTP site"), ftplink_cmd },*/
 	{ GNOME_APP_UI_ITEM, N_("_Copy..."),           N_("Copy files"), copy_cmd, NULL},
 	{ GNOME_APP_UI_ITEM, N_("_Delete..."),         N_("Delete files from disk"), delete_cmd },
 	{ GNOME_APP_UI_ITEM, N_("_Move..."),           N_("Rename or move files"), ren_cmd },
-/*	{ GNOME_APP_UI_ITEM, N_("_Make directory..."), N_("Creates a new directory"), mkdir_cmd },*/
 	{ GNOME_APP_UI_SEPARATOR },
 	{ GNOME_APP_UI_ITEM, N_("C_lose"),             N_("Close this panel"), gnome_close_panel, NULL,
 	  NULL, GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_CLOSE,
@@ -431,19 +441,108 @@ static TbItems tb_items[] =
 
 static GtkWidget *create_toolbar (GtkWidget * window, GtkWidget *widget)
 {
-    GtkWidget *toolbar;
-    TbItems *t;
-    toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
-    for (t = &tb_items[0]; t->text; t++) {
-	t->widget = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-					     t->text,
+	GtkWidget *toolbar;
+	TbItems *t;
+	
+	toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+	
+	for (t = &tb_items[0]; t->text; t++){
+		t->widget = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
+						     t->text,
 					     t->tooltip,
-					     0,
-			     gnome_stock_pixmap_widget (window, t->icon),
-					     t->cb,
-					     t->cb ? widget : 0);
-    }
-    return toolbar;
+						     0,
+						     gnome_stock_pixmap_widget (window, t->icon),
+						     t->cb,
+						     t->cb ? widget : 0);
+	}
+	return toolbar;
+}
+
+static void
+create_new_menu_from (GnomeApp *app, WPanel *panel, char *file)
+{
+	gint pos;
+	GtkWidget *shell = NULL;
+	DIR *dir;
+	struct stat filedata;
+	struct dirent *dirstruc;
+	GnomeDesktopEntry *gde;
+	GtkWidget *menu;
+	char *file2;
+	
+	dir = opendir (file);
+	if (dir == NULL)
+		return;
+	
+	shell = gnome_app_find_menu_pos (app->menubar, _("File/New/Directory..."), &pos);
+	menu = gtk_menu_item_new ();
+	gtk_widget_show (menu);
+	gtk_menu_shell_insert (GTK_MENU_SHELL (shell), menu, pos++);
+
+	if (shell == NULL){
+		closedir (dir);
+		return;
+	}
+
+	while ((dirstruc = readdir (dir)) != NULL){
+		if (dirstruc->d_name[0] == '.')
+			continue;
+
+		file2 = g_concat_dir_and_file (file, dirstruc->d_name);
+
+		if ((stat (file2, &filedata) != -1) && (S_ISREG (filedata.st_mode))){
+			char *test;
+			
+			gde = gnome_desktop_entry_load (file2);
+
+			test = rindex(dirstruc->d_name, '.');
+
+			if (test == NULL || gde == NULL || strcmp (test, ".desktop")){
+				g_free (file2);
+				continue;
+			}
+
+			if (!gnome_is_program_in_path (gde->tryexec)){
+				g_free (file2);
+				g_print ("yes!\n");
+				continue;
+			}
+			
+			menu = gtk_menu_item_new_with_label (gde->name);
+			gtk_widget_show (menu);
+			gtk_menu_shell_insert (GTK_MENU_SHELL (shell), menu, pos++);
+
+			/* This is really bad, but it works. */
+			if (gde->comment)
+				gtk_object_set_data (GTK_OBJECT (menu), "apphelper_statusbar_hint",
+						     gde->comment);
+			gtk_signal_connect (GTK_OBJECT (menu), "activate", GTK_SIGNAL_FUNC (gnome_run_new),
+					    gde);
+		}
+		g_free (file2);
+	}
+}
+
+/**
+ * create_new_menu:
+ *
+ * Creates the child New menu items
+ */
+static void
+create_new_menu (GnomeApp *app, WPanel *panel)
+{
+	gchar *file, *file2;
+
+	file = gnome_unconditional_datadir_file ("mc/templates");
+	create_new_menu_from (app, panel, file);
+	
+	file2 = gnome_datadir_file ("mc/templates");
+	if (file2 != NULL){
+		if (strcmp (file, file2) != 0)
+			create_new_menu_from (app, panel, file2);
+	}
+	g_free (file);
+	g_free (file2);
 }
 
 WPanel *
@@ -469,11 +568,18 @@ create_container (Dlg_head *h, char *name, char *geometry)
 		gtk_widget_set_uposition (GTK_WIDGET (app), xpos, ypos);
 
 	panel = panel_new (name);
+
+	
 	vbox = gtk_vbox_new (FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 0);
 	gnome_app_set_contents (GNOME_APP (app), vbox);
 	gnome_app_create_menus_with_data (GNOME_APP (app), gnome_panel_menu, panel);
+	create_new_menu (GNOME_APP (app), panel);
 
+	panel->ministatus = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER));
+	gnome_app_set_statusbar(GNOME_APP (app), GTK_WIDGET(panel->ministatus));
+
+	gnome_app_install_menu_hints (GNOME_APP (app), gnome_panel_menu);
 	/*
 	 * I am trying to unclutter the screen, so this toolbar is gone now
 	 */
