@@ -38,6 +38,9 @@
 #include "../vfs/vfs.h"
 #include <gdk/gdkprivate.h>
 
+/* Whether to display the tree view on the left */
+int tree_panel_visible = -1;
+
 /* The pixmaps */
 #include "directory.xpm"
 #include "link.xpm"
@@ -78,6 +81,7 @@ GtkWidget *drag_multiple_ok  = NULL;
 
 static void panel_file_list_configure_contents (GtkWidget *sw, WPanel *panel, int main_width, int height);
 
+
 #define CLIST_FROM_SW(panel_list) GTK_CLIST (GTK_BIN (panel_list)->child)
 
 
@@ -226,9 +230,12 @@ panel_fill_panel_icons (WPanel *panel)
 
 	for (i = 0; i < top; i++){
 		file_entry *fe = &panel->dir.list [i];
-
+		int p;
+		
 		image = gicon_get_icon_for_file (fe);
-		gnome_icon_list_append_imlib (icons, image, fe->fname);
+		p = gnome_icon_list_append_imlib (icons, image, fe->fname);
+		if (fe->f.marked)
+			gnome_icon_list_select_icon (icons, p);
 	}
 	/* This is needed as the gtk_clist_append changes selected under us :-( */
 	panel->selected = selected;
@@ -249,7 +256,13 @@ x_fill_panel (WPanel *panel)
 
 	gtk_signal_handler_block_by_data (GTK_OBJECT (panel->tree), panel);
 
-	gtk_dtree_select_dir (GTK_DTREE (panel->tree), panel->cwd);
+	if (vfs_current_is_local ()){
+		char buffer [MC_MAXPATHLEN];
+
+		get_current_wd (buffer, sizeof (buffer)-1);
+		gtk_dtree_select_dir (GTK_DTREE (panel->tree), buffer);
+	} else
+		gtk_dtree_select_dir (GTK_DTREE (panel->tree), panel->cwd);
 
 	gtk_signal_handler_unblock_by_data (GTK_OBJECT (panel->tree), panel);
 }
@@ -2041,14 +2054,24 @@ create_and_setup_pane (WPanel *panel)
 	GtkWidget *pane;
 	GtkWidget *tree = panel->tree;
 	GdkFont *tree_font = tree->style->font;
+	int size;
 
 	pane = gtk_hpaned_new ();
 
+	if (tree_panel_visible == -1)
+		 size = 20 * gdk_string_width (tree_font, "W");
+	else {
+		if (tree_panel_visible)
+			size = tree_panel_visible;
+		else
+			size = 0;
+	}
+	
 	/*
 	 * Hack: set the default startup size for the pane without
 	 * using _set_usize which would set the minimal size
 	 */
-	GTK_PANED (pane)->child1_size = 20 * gdk_string_width (tree_font, "W");
+	GTK_PANED (pane)->child1_size = size;
 	GTK_PANED (pane)->position_set = TRUE;
 
 	gtk_widget_show (pane);
@@ -2090,6 +2113,8 @@ button_switch_to (char **icon, GtkSignalFunc fn, void *closure)
 static void
 do_switch_to_iconic (GtkWidget *widget, WPanel *panel)
 {
+	if (GTK_TOGGLE_BUTTON (widget)->active == FALSE)
+		return;
 	if (panel->list_type == list_icons)
 		return;
 	panel->list_type = list_icons;
@@ -2100,6 +2125,8 @@ do_switch_to_iconic (GtkWidget *widget, WPanel *panel)
 static void
 do_switch_to_brief_listing (GtkWidget *widget, WPanel *panel)
 {
+	if (GTK_TOGGLE_BUTTON (widget)->active == FALSE)
+		return;
 	if (panel->list_type == list_brief)
 		return;
 	panel->list_type = list_brief;
@@ -2110,6 +2137,8 @@ do_switch_to_brief_listing (GtkWidget *widget, WPanel *panel)
 static void
 do_switch_to_full_listing (GtkWidget *widget, WPanel *panel)
 {
+	if (GTK_TOGGLE_BUTTON (widget)->active == FALSE)
+		return;
 	if (panel->list_type == list_full)
 		return;
 	panel->list_type = list_full;
@@ -2120,6 +2149,8 @@ do_switch_to_full_listing (GtkWidget *widget, WPanel *panel)
 static void
 do_switch_to_custom_listing (GtkWidget *widget, WPanel *panel)
 {
+	if (GTK_TOGGLE_BUTTON (widget)->active == FALSE)
+		return;
 	if (panel->list_type == list_user)
 		return;
 	panel->list_type = list_user;
@@ -2127,37 +2158,71 @@ do_switch_to_custom_listing (GtkWidget *widget, WPanel *panel)
 	panel_update_contents (panel);
 }
 
-static GtkWidget *
-button_switch_to_icon (WPanel *panel)
+static void
+go_home (GtkWidget *widget, WPanel *panel)
 {
-	return button_switch_to (listing_iconic_xpm, GTK_SIGNAL_FUNC (do_switch_to_iconic), panel);
+	do_panel_cd (panel, "~", cd_exact);
+}
+static GnomeUIInfo viewbar[] = {
+	{ GNOME_APP_UI_ITEM, N_("Icon"), N_("Switch view to an Icon view"), do_switch_to_iconic, NULL, NULL, \
+		GNOME_APP_PIXMAP_DATA, listing_iconic_xpm, 0, (GdkModifierType) 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Brief"), N_("Switch view to show just file name and type"), do_switch_to_brief_listing, NULL, NULL, \
+		GNOME_APP_PIXMAP_DATA, listing_brief_list_xpm, 0, (GdkModifierType) 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Detailed"), N_("Switch view to show detailed file statistics"), do_switch_to_full_listing, NULL, NULL, \
+		GNOME_APP_PIXMAP_DATA, listing_list_xpm, 0, (GdkModifierType) 0, NULL },
+	{ GNOME_APP_UI_ITEM, N_("Custom"), N_("Switch view to show custom determined statistics"), do_switch_to_custom_listing, NULL, NULL, \
+		GNOME_APP_PIXMAP_DATA, listing_custom_xpm, 0, (GdkModifierType) 0, NULL },
+	GNOMEUIINFO_END
+};
+static GnomeUIInfo toolbar[] = {
+	GNOMEUIINFO_ITEM_STOCK (N_("Back"), N_("Go to the previously visited directory"),
+				panel_back, GNOME_STOCK_PIXMAP_BACK),
+	GNOMEUIINFO_ITEM_STOCK (N_("Up"), N_("Go up a level in the directory heirarchy"),
+				panel_up, GNOME_STOCK_PIXMAP_UP),
+	GNOMEUIINFO_ITEM_STOCK (N_("Forward"), N_("Go to the next directory"),
+				panel_fwd, GNOME_STOCK_PIXMAP_FORWARD),
+	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_ITEM_STOCK (N_("Home"), N_("Go to your home directory"),
+				go_home, GNOME_STOCK_PIXMAP_HOME),
+	GNOMEUIINFO_SEPARATOR,
+	GNOMEUIINFO_RADIOLIST(viewbar),
+	GNOMEUIINFO_END
+};
+static void
+do_ui_signal_connect (GnomeUIInfo *uiinfo, gchar *signal_name, 
+		GnomeUIBuilderData *uibdata)
+{
+	if (uiinfo->moreinfo)
+		gtk_signal_connect (GTK_OBJECT (uiinfo->widget), 
+				    signal_name, uiinfo->moreinfo, uibdata->data ? 
+				    uibdata->data : uiinfo->user_data);
 }
 
-static GtkWidget *
-button_switch_to_full_listing (WPanel *panel)
+
+
+static void
+tree_size_allocate (GtkWidget *widget, GtkAllocation *allocation, WPanel *panel)
 {
-	return button_switch_to (listing_list_xpm, GTK_SIGNAL_FUNC (do_switch_to_full_listing), panel);
-}
-static GtkWidget *
-button_switch_to_brief_listing (WPanel *panel)
-{
-	return button_switch_to (listing_brief_list_xpm, GTK_SIGNAL_FUNC (do_switch_to_brief_listing), panel);
-}
-static GtkWidget *
-button_switch_to_custom_listing (WPanel *panel)
-{
-	return button_switch_to (listing_custom_xpm, GTK_SIGNAL_FUNC (do_switch_to_custom_listing), panel);
+	GtkWidget *tree = panel->tree;
+	GdkFont *tree_font = tree->style->font;
+	
+	if (allocation->width <= 0){
+		tree_panel_visible = 0;
+	} else {
+		tree_panel_visible = allocation->width;
+	}
+	save_setup ();
 }
 
 void
 x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 {
 	GtkWidget *status_line, *filter, *vbox, *ministatus_box;
-	GtkWidget *frame, *cwd, *back_p, *fwd_p, *up_p;
+	GtkWidget *cwd, *back_p, *fwd_p, *up_p;
 	GtkWidget *hbox, *evbox, *dock, *box;
-	GtkWidget *table_frame;
-	int display;
-	
+	GnomeUIBuilderData uibdata;
+
+
 	panel->xwindow = gtk_widget_get_toplevel (GTK_WIDGET (panel->widget.wdata));
 
 	panel->table = gtk_table_new (2, 1, 0);
@@ -2211,7 +2276,9 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	 */
 	panel->pane = create_and_setup_pane (panel);
 	gtk_paned_add1 (GTK_PANED (panel->pane), panel->tree_scrolled_window);
-
+	gtk_signal_connect (GTK_OBJECT (panel->tree_scrolled_window), "size_allocate",
+			    GTK_SIGNAL_FUNC (tree_size_allocate), panel);
+	
 	/*
 	 * Filter
 	 */
@@ -2224,66 +2291,83 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 
 	/* We do not want the focus by default  (and the previos add_widget just gave it to us) */
 	h->current = h->current->prev;
-
+	
 	/*
-	 * History buttons, and updir.
+	 * We go through a lot of pain, wrestling with gnome_app* and gmc's @#$&*#$ internal structure and
+	 * make the #@$*&@#$ toolbars here...
 	 */
-	back_p = gnome_stock_pixmap_widget_new (panel->xwindow, GNOME_STOCK_MENU_BACK);
-	fwd_p  = gnome_stock_pixmap_widget_new (panel->xwindow, GNOME_STOCK_MENU_FORWARD);
-	up_p   = gnome_stock_pixmap_widget_new (panel->xwindow, GNOME_STOCK_MENU_UP);
-	panel->up_b     = gtk_button_new ();
-	panel->back_b   = gtk_button_new ();
-	panel->fwd_b    = gtk_button_new ();
-	gtk_container_add (GTK_CONTAINER (panel->back_b), back_p);
-	gtk_container_add (GTK_CONTAINER (panel->fwd_b), fwd_p);
-	gtk_container_add (GTK_CONTAINER (panel->up_b), up_p);
-	gtk_signal_connect (GTK_OBJECT (panel->back_b), "clicked", GTK_SIGNAL_FUNC(panel_back), panel);
-	gtk_signal_connect (GTK_OBJECT (panel->fwd_b), "clicked", GTK_SIGNAL_FUNC(panel_fwd), panel);
-	gtk_signal_connect (GTK_OBJECT (panel->up_b), "clicked", GTK_SIGNAL_FUNC(panel_up), panel);
+
+	status_line = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+	uibdata.connect_func = do_ui_signal_connect;
+	uibdata.data = panel;
+	uibdata.is_interp = FALSE;
+	uibdata.relay_func = NULL;
+	uibdata.destroy_func = NULL;
+
+	gnome_app_fill_toolbar_custom (GTK_TOOLBAR (status_line), toolbar,  &uibdata, NULL);
+	gnome_app_add_toolbar (GNOME_APP (panel->xwindow),
+			       GTK_TOOLBAR (status_line),
+			       "gmc-toolbar0",
+			       GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
+			       GNOME_DOCK_TOP,
+			       2, 0, 0);
+	
+	panel->back_b   = toolbar[0].widget;
+	panel->up_b     = toolbar[1].widget;
+	panel->fwd_b    = toolbar[2].widget;
 	panel_update_marks (panel);
-
-
 	/*
-	 * Here we make the toolbars
+	 * Here we make the view buttons.
 	 */
 	status_line = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
 	gtk_container_set_border_width (GTK_CONTAINER (status_line), 3);
 	evbox = gtk_event_box_new ();
 	hbox = gtk_hbox_new (FALSE, 2);
 	gtk_container_add (GTK_CONTAINER (evbox), hbox);
-	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("Location:")), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), cwd, FALSE, FALSE, 0);
 	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
 				   evbox,
 				   NULL, NULL);
-	gtk_toolbar_append_space (GTK_TOOLBAR (status_line));
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   panel->back_b,
-				   "Go to the previous directory.", NULL);
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   panel->up_b,
-				   "Go up a level in the directory heirarchy.", NULL);
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   panel->fwd_b,
-				   "Go to the next directory.", NULL);
-	gtk_toolbar_append_space (GTK_TOOLBAR (status_line));
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   button_switch_to_icon (panel),
-				   "Icon view.", NULL);
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   button_switch_to_brief_listing (panel),
-				   "Brief view.", NULL);
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   button_switch_to_full_listing (panel),
-				   "Detailed view.", NULL);
-	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
-				   button_switch_to_custom_listing (panel),
-				   "Custom view.", NULL);
-	dock =  gnome_dock_item_new ("gmc-toolbar", GNOME_DOCK_ITEM_BEH_EXCLUSIVE | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("Location:")), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), cwd, FALSE, FALSE, 0);
+	dock =  gnome_dock_item_new ("gmc-toolbar1", GNOME_DOCK_ITEM_BEH_EXCLUSIVE | GNOME_DOCK_ITEM_BEH_NEVER_VERTICAL);
 	gtk_container_add (GTK_CONTAINER(dock),status_line);
 	gnome_dock_add_item (GNOME_DOCK(GNOME_APP (panel->xwindow)->dock),
 			     GNOME_DOCK_ITEM (dock), GNOME_DOCK_TOP, 1, 0, 0, FALSE);
+	
 	gtk_widget_show_all (dock);
+
+#if 0
+	status_line = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   panel->back_b,
+				   "Go to the previous directory", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   panel->up_b,
+				   "Go up a level in the directory heirarchy", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   panel->fwd_b,
+				   "Go to the next directory", NULL);
+	gtk_toolbar_append_space (GTK_TOOLBAR (status_line));
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   button_switch_to_icon (panel),
+				   "Icon view", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   button_switch_to_brief_listing (panel),
+				   "Brief view", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   button_switch_to_full_listing (panel),
+				   "Detailed view", NULL);
+	gtk_toolbar_append_widget (GTK_TOOLBAR (status_line),
+				   button_switch_to_custom_listing (panel),
+				   "Custom view", NULL);
+	gnome_app_add_toolbar (GNOME_APP (panel->xwindow),
+			       GTK_TOOLBAR (status_line),
+			       "gmc-toolbar2",
+			       GNOME_DOCK_ITEM_BEH_EXCLUSIVE,
+			       GNOME_DOCK_TOP,
+			       2, 0, 0);
+	gtk_widget_show_all (status_line);
+#endif
 	panel->view_table = gtk_table_new (1, 1, 0);
 	gtk_widget_show (panel->view_table);
 
@@ -2291,13 +2375,12 @@ x_create_panel (Dlg_head *h, widget_data parent, WPanel *panel)
 	/*
 	 * The status bar.
 	 */
-	panel->ministatus = GNOME_APPBAR(gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER));
-	gnome_app_set_statusbar(GNOME_APP (panel->xwindow), GTK_WIDGET(panel->ministatus));
 	ministatus_box = gtk_frame_new (NULL);
 	gtk_frame_set_shadow_type (GTK_FRAME(ministatus_box), GTK_SHADOW_IN);
       
 	panel->status = gtk_label_new (_("Show all files"));
 	gtk_misc_set_alignment (GTK_MISC (panel->status), 0.0, 0.0);
+	gtk_misc_set_padding (GTK_MISC (panel->status), 2, 0);
       
 	gtk_box_pack_start (GTK_BOX (panel->ministatus), ministatus_box, FALSE, FALSE, 0);
 	gtk_container_add (GTK_CONTAINER(ministatus_box), panel->status);
