@@ -224,7 +224,6 @@ gtk_dtree_do_select_dir (GtkDTree *dtree, char *path)
 {
 	GtkCTreeNode *current_node;
 	char *s, *current, *npath;
-	char *request;
 	
 	g_return_val_if_fail (dtree != NULL, FALSE);
 	g_return_val_if_fail (GTK_IS_DTREE (dtree), FALSE);
@@ -267,7 +266,8 @@ gtk_dtree_do_select_dir (GtkDTree *dtree, char *path)
         g_free (npath);
 	
 	if (current_node){
-		if (!gtk_ctree_node_is_visible (GTK_CTREE (dtree), current_node)){
+		gtk_ctree_select (GTK_CTREE (dtree), current_node);
+		if (gtk_ctree_node_is_visible (GTK_CTREE (dtree), current_node) != GTK_VISIBILITY_FULL){
 			gtk_ctree_node_moveto (GTK_CTREE (dtree), current_node, 0, 0.5, 0.0);
 		}
 
@@ -504,8 +504,6 @@ gtk_dtree_dirty_notify (int state)
 static void
 gtk_dtree_init (GtkDTree *dtree)
 {
-	static int tree_inited;
-	
 	dtree->current_path = NULL;
 	dtree->auto_expanded_nodes = NULL;
 
@@ -534,7 +532,7 @@ gtk_dtree_construct (GtkDTree *dtree)
 	gtk_clist_columns_autosize (clist);
 	
 	gtk_ctree_set_line_style (ctree, GTK_CTREE_LINES_DOTTED);
-	gtk_ctree_set_reorderable (ctree, FALSE);
+	gtk_clist_set_reorderable (GTK_CLIST (ctree), FALSE);
 
 	gdk_dtree_load_pixmaps (dtree);
 	gtk_dtree_load_root_tree (dtree);
