@@ -1,3 +1,23 @@
+/* gtkedit.c -
+ front end for gtk/gnome version
+
+   Copyright (C) 1996, 1997 the Free Software Foundation
+
+   Authors: 1996, 1997 Paul Sheer
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; either version 2 of the License, or
+   (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #define _GTK_EDIT_C
 
@@ -20,6 +40,7 @@
 #define GDK_FONT_XFONT(font) (((GdkFontPrivate*) font)->xfont)
 
 int edit_key_emulation = 0;
+int column_highlighting = 0;
 
 static GtkWidgetClass *parent_class = NULL;
 
@@ -31,6 +52,7 @@ static void gtk_edit_set_position (GtkEditable *editable, gint position);
 void edit_mouse_mark (WEdit * edit, XEvent * event, int double_click);
 void edit_move_to_prev_col (WEdit * edit, long p);
 int edit_load_file_from_filename (WEdit *edit, char *exp);
+static void gtk_edit_set_selection (GtkEditable * editable, gint start, gint end);
 
 
 guchar gtk_edit_font_width_per_char[256];
@@ -278,11 +300,11 @@ static void gtk_edit_destroy (GtkObject * object)
 	gtk_timeout_remove (edit->timer);
 	edit->timer = 0;
     }
-    if (edit->funcs) {
-	free (edit->funcs);
+    edit_clean (edit->editor);
+    if (edit->editor) {
+	free (edit->editor);
+	edit->editor = NULL;
     }
-    edit_destroy_callback (edit);
-
     GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
@@ -505,15 +527,84 @@ static gint
     return FALSE;
 }
 
-static gint
- gtk_edit_button_press_release (GtkWidget * widget,
-				GdkEventButton * event)
+void edit_update_screen (WEdit * e)
+{
+    if (!e)
+	return;
+    if (!e->force)
+	return;
+
+    edit_scroll_screen_over_cursor (e);
+    edit_update_curs_row (e);
+    edit_update_curs_col (e);
+#if 0
+    update_scroll_bars (e);
+#endif
+    edit_status (e);
+
+    if (e->force & REDRAW_COMPLETELY)
+	e->force |= REDRAW_PAGE;
+
+/* pop all events for this window for internal handling */
+    if (e->force & (REDRAW_CHAR_ONLY | REDRAW_COMPLETELY)) {
+	edit_render_keypress (e);
+#if 0
+    } else if (CCheckWindowEvent (e->widget->winid, ButtonPressMask | ButtonReleaseMask | ButtonMotionMask, 0)
+	       || CKeyPending ()) {
+	e->force |= REDRAW_PAGE;
+	return;
+#endif
+    } else {
+	edit_render_keypress (e);
+    }
+}
+
+
+void gtk_edit_mouse_redraw (WEdit * edit, long click)
+{
+    edit->force |= REDRAW_PAGE | REDRAW_LINE;
+    edit_update_curs_row (edit);
+    edit_update_curs_col (edit);
+    edit->prev_col = edit_get_col (edit);
+    edit_update_screen (edit);
+    edit->search_start = click;
+}
+
+/* returns the position in the edit buffer of a window click */
+static long edit_get_click_pos (WEdit * edit, int x, int y)
+{
+    long click;
+/* (1) goto to left margin */
+    click = edit_bol (edit, edit->curs1);
+
+/* (1) move up or down */
+    if (y > (edit->curs_row + 1))
+	click = edit_move_forward (edit, click, y - (edit->curs_row + 1), 0);
+    if (y < (edit->curs_row + 1))
+	click = edit_move_backward (edit, click, (edit->curs_row + 1) - y);
+
+/* (3) move right to x pos */
+    click = edit_move_forward3 (edit, click, x - edit->start_col - 1, 0);
+    return click;
+}
+
+static void edit_translate_xy (int xs, int ys, int *x, int *y)
+{
+    *x = xs - EDIT_TEXT_HORIZONTAL_OFFSET;
+    *y = (ys - EDIT_TEXT_VERTICAL_OFFSET - option_text_line_spacing / 2 - 1) / FONT_PIX_PER_LINE + 1;
+}
+
+static gint gtk_edit_button_press_release (GtkWidget * widget,
+					   GdkEventButton * event)
 {
-    int double_click = 0;
-    XEvent xevent;
-    GtkEdit *edit;
     GtkEditable *editable;
+    GtkEdit *edit;
+    WEdit *e;
     static GdkAtom ctext_atom = GDK_NONE;
+    int x_text = 0, y_text = 0;
+    long mouse_pos;
+    static long button_down_pos;
+    static long dragging = 0;
 
     g_return_val_if_fail (widget != NULL, FALSE);
     g_return_val_if_fail (GTK_IS_EDIT (widget), FALSE);
@@ -523,37 +614,73 @@ static gint
 	ctext_atom = gdk_atom_intern ("COMPOUND_TEXT", FALSE);
 
     edit = GTK_EDIT (widget);
+    e = edit->editor;
     editable = GTK_EDITABLE (widget);
 
     if (!GTK_WIDGET_HAS_FOCUS (widget))
 	if (!(event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)))
 	    gtk_widget_grab_focus (widget);
 
+    edit_translate_xy (event->x, event->y, &x_text, &y_text);
+    mouse_pos = edit_get_click_pos (e, x_text, y_text);
+
     switch (event->type) {
+    case GDK_BUTTON_RELEASE:
+	if (!dragging)
+	    return FALSE;
     case GDK_MOTION_NOTIFY:
-	xevent.type = MotionNotify;
+	if (mouse_pos == button_down_pos)
+	    return FALSE;
+	dragging = 1;
+	edit_cursor_move (e, mouse_pos - e->curs1);
+	gtk_edit_set_selection (GTK_EDITABLE (widget), mouse_pos, button_down_pos);
+	gtk_edit_mouse_redraw (e, mouse_pos);
 	break;
     case GDK_BUTTON_PRESS:
-	xevent.type = ButtonPress;
+	dragging = 0;
+	if (event->button == 2) {
+	    edit_cursor_move (e, mouse_pos - e->curs1);
+	    editable->current_pos = mouse_pos;
+	    gtk_selection_convert (GTK_WIDGET (edit), GDK_SELECTION_PRIMARY,
+				   ctext_atom, event->time);
+	    gtk_edit_mouse_redraw (e, mouse_pos);
+	    editable->selection_start_pos = e->mark1;
+	    editable->selection_end_pos = e->mark2;
+	    return FALSE;
+	}
+	button_down_pos = mouse_pos;
+#if 0
+	if (editable->has_selection)
+	    if (mouse_pos >= editable->selection_start_pos
+		&& mouse_pos < editable->selection_end_pos)
+		gtk_edit_set_selection (GTK_EDITABLE (widget), mouse_pos, mouse_pos);
+#endif
+	edit_cursor_move (e, mouse_pos - e->curs1);
+	gtk_edit_mouse_redraw (e, mouse_pos);
 	break;
     case GDK_2BUTTON_PRESS:
-	xevent.type = ButtonPress;
-	double_click = 1;
-	break;
-    case GDK_BUTTON_RELEASE:
-	xevent.type = ButtonRelease;
+	dragging = 0;
+	edit_cursor_move (e, mouse_pos - e->curs1);
+	edit_right_word_move (e);
+	mouse_pos = e->curs1;
+	edit_left_word_move (e);
+	button_down_pos = e->curs1;
+	gtk_edit_set_selection (GTK_EDITABLE (widget), mouse_pos, button_down_pos);
+	gtk_edit_mouse_redraw (e, mouse_pos);
 	break;
     case GDK_3BUTTON_PRESS:
+	dragging = 0;
+	mouse_pos = edit_bol (e, mouse_pos);
+	edit_cursor_move (e, mouse_pos - e->curs1);
+	button_down_pos = edit_eol (e, mouse_pos) + 1;
+	gtk_edit_set_selection (GTK_EDITABLE (widget), mouse_pos, button_down_pos);
+	gtk_edit_mouse_redraw (e, mouse_pos);
+	break;
     default:
-	return FALSE;
+	dragging = 0;
 	break;
     }
-    xevent.xbutton.window = (guint32) event->window;
-    xevent.xbutton.x = event->x;
-    xevent.xbutton.y = event->y;
-    xevent.xbutton.state = event->state;
-
-    edit_mouse_mark (edit->editor, &xevent, double_click);
+    editable->current_pos = e->curs1;
     return FALSE;
 }
 
@@ -609,22 +736,19 @@ short key_sym_mod (gint key, gint state)
     return key;
 }
 
-static gint
- gtk_edit_key_press (GtkWidget * widget,
-		     GdkEventKey * event)
+static gint gtk_edit_key_press (GtkWidget * widget, GdkEventKey * event)
 {
-    GtkEdit *edit;
     GtkEditable *editable;
+    GtkEdit *edit;
+    WEdit *e;
     gint command = 0, insert = -1, r = 0;
     guint key, state;
-
     g_return_val_if_fail (widget != NULL, FALSE);
     g_return_val_if_fail (GTK_IS_EDIT (widget), FALSE);
     g_return_val_if_fail (event != NULL, FALSE);
-
     edit = GTK_EDIT (widget);
     editable = GTK_EDITABLE (widget);
-
+    e = edit->editor;
     if (!edit_translate_key (0, event->keyval, event->state, &command, &insert)) {
 	return FALSE;
     }
@@ -632,17 +756,20 @@ static gint
     state = event->state;
     if (!command && insert < 0) {	/* no translation took place, so lets see if we have a macro */
 	if ((key == GDK_r || key == GDK_R) && (state & ControlMask)) {
-	    command = edit->editor->macro_i < 0 ? CK_Begin_Record_Macro : CK_End_Record_Macro;
+	    command = e->macro_i < 0 ? CK_Begin_Record_Macro : CK_End_Record_Macro;
 	} else {
 	    command = key_sym_mod (key, state);
 	    if (command > 0)
 		command = CK_Macro (command);
 	}
     }
-    r = edit_execute_key_command (edit->editor, command, insert);
+    r = edit_execute_key_command (e, command, insert);
     if (r)
-	edit_update_screen (edit->editor);
-
+	edit_update_screen (e);
+    editable->selection_start_pos = e->mark1;
+    editable->selection_end_pos = ((e->mark2 < 0) ? e->curs1 : e->mark2);
+    editable->has_selection = editable->selection_start_pos != editable->selection_end_pos;
+    editable->current_pos = e->curs1;
     return r;
 }
 
@@ -672,6 +799,56 @@ static void get_home_dir (void)
     abort ();
 }
 
+static gchar *gtk_edit_get_chars (GtkEditable * editable,
+				  gint start_pos,
+				  gint end_pos)
+{
+    GtkEdit *edit;
+    gchar *retval;
+    int i;
+    g_return_val_if_fail (editable != NULL, NULL);
+    g_return_val_if_fail (GTK_IS_EDIT (editable), NULL);
+    edit = GTK_EDIT (editable);
+    if (end_pos < 0)
+	end_pos = edit->editor->last_byte;
+    if ((start_pos < 0) ||
+	(end_pos > edit->editor->last_byte) ||
+	(end_pos < start_pos))
+	return 0;
+    retval = malloc (end_pos - start_pos + 1);
+    retval[end_pos - start_pos] = '\0';
+    for (i = 0; start_pos < end_pos; start_pos++, i++)
+	retval[i] = (gchar) edit_get_byte (edit->editor, start_pos);
+    return retval;
+}
+
+static void gtk_edit_set_selection (GtkEditable * editable,
+				    gint start,
+				    gint end)
+{
+    GtkEdit *edit = GTK_EDIT (editable);
+    WEdit *e;
+    e = edit->editor;
+    if (end < 0)
+	end = edit->editor->last_byte;
+    editable->selection_start_pos = e->mark1 = MIN (start, end);
+    editable->selection_end_pos = e->mark2 = MAX (start, end);
+    editable->has_selection = (e->mark2 != e->mark1);
+    gtk_edit_mouse_redraw (e, e->curs1);
+    gtk_editable_claim_selection (editable, TRUE, GDK_CURRENT_TIME);
+}
+
+static void gtk_edit_insert_text (GtkEditable * editable,
+				  const gchar * new_text,
+				  gint new_text_length,
+				  gint * position)
+{
+    GtkEdit *edit = GTK_EDIT (editable);
+    edit_cursor_move (edit->editor, *position - edit->editor->curs1);
+    while (new_text_length--)
+	edit_insert_ahead (edit->editor, new_text[new_text_length]);
+    *position = edit->editor->curs1;
+}
 
 static void gtk_edit_class_init (GtkEditClass * class)
 {
@@ -707,19 +884,19 @@ static void gtk_edit_class_init (GtkEditClass * class)
     widget_class->focus_in_event = 0;
     widget_class->focus_out_event = 0;
 
-#if 0
     editable_class->insert_text = gtk_edit_insert_text;
+#if 0
     editable_class->delete_text = gtk_edit_delete_text;
     editable_class->update_text = gtk_edit_update_text;
-    editable_class->get_chars = gtk_edit_get_chars;
-    editable_class->set_selection = gtk_edit_set_selection;
-#endif
-    editable_class->set_position = gtk_edit_set_position;
-
-    editable_class->insert_text = 0;
+#else
     editable_class->delete_text = 0;
     editable_class->update_text = 0;
-    editable_class->get_chars = 0;
+#endif
+
+    editable_class->get_chars = gtk_edit_get_chars;
+    editable_class->set_selection = gtk_edit_set_selection;
+    editable_class->set_position = gtk_edit_set_position;
+
 
 #if 0
     editable_class->set_position = 0;
@@ -728,8 +905,6 @@ static void gtk_edit_class_init (GtkEditClass * class)
     get_home_dir ();
 }
 
-extern struct mouse_funcs edit_mouse_funcs;
-
 static void gtk_edit_init (GtkEdit * edit)
 {
     static made_directory = 0;
@@ -743,7 +918,6 @@ static void gtk_edit_init (GtkEdit * edit)
     edit->menubar = 0;
     edit->status = 0;
     edit->options = 0;
-    edit->funcs = mouse_funcs_new (edit->editor, &edit_mouse_funcs);
 
     gtk_edit_configure_font_dimensions (edit);
 
@@ -812,7 +986,7 @@ char *gtk_edit_dialog_get_load_file (guchar * dir, guchar * def, guchar * title)
 void gtk_edit_dialog_message (guchar * heading, char *fmt,...)
 {
     gchar s[8192];
-    va_list *ap;
+    va_list ap;
     va_start (ap, fmt);
     vsprintf (s, fmt, ap);
     va_end (ap);
@@ -830,7 +1004,7 @@ int gtk_edit_dialog_query (guchar * heading, guchar * first,...)
     char *buttons[16];
     char s[1024], *r;
     int n;
-    va_list *ap;
+    va_list ap;
     va_start (ap, first);
     n = 0;
     while ((buttons[n++] = va_arg (ap, char *)) && n < 15);
@@ -866,7 +1040,7 @@ int gtk_edit_dialog_query (guchar * heading, guchar * first,...)
 void gtk_edit_dialog_error (guchar * heading, char *fmt, ...)
 {
     gchar s[8192];
-    va_list *ap;
+    va_list ap;
     va_start (ap, fmt);
     vsprintf (s, fmt, ap);
     va_end (ap);
@@ -1171,7 +1345,7 @@ int edit (const char *file, int line)
 	{
 	    {
 		GNOME_APP_UI_ITEM,
-		N_ ("About..."), N_ ("Info about GNOME hello"),
+		N_ ("About..."), N_ ("Info about Mcedit"),
 		about_cb, NULL, NULL,
 		GNOME_APP_PIXMAP_STOCK, GNOME_STOCK_MENU_ABOUT,
 		0, (GdkModifierType) 0, NULL
@@ -1193,7 +1367,8 @@ int edit (const char *file, int line)
 	};
 
 	gtk_widget_realize (app);
-	statusbar = gtk_label_new (file ? file : "<no-filename>");
+	statusbar = gtk_entry_new ();
+	gtk_entry_set_editable (GTK_ENTRY (statusbar), 0);
 	gtk_widget_set_usize (app, 400, 400);
 	gnome_app_create_menus (GNOME_APP (app), main_menu);
 	gnome_app_set_contents (GNOME_APP (app), edit);
@@ -1213,7 +1388,7 @@ int edit (const char *file, int line)
 		    return 1;
 		}
 	gtk_edit_set_cursor_line (edit, line);
-	gtk_widget_show (app);
+	gtk_widget_show_all (app);
 	gtk_widget_grab_focus (edit);
     }
     return 0;
