@@ -123,15 +123,6 @@ struct {
 #ifdef HAVE_X
 static void chmod_toggle_select (void)
 {
-#ifdef HAVE_TK
-    char *wn = (char *) ch_dlg->current->widget->wdata;
-    int  id = ch_dlg->current->dlg_id -BUTTONS + single_set * 2;
-    
-    check_perm [id].selected ^= 1;
-
-    tk_evalf ("%s configure -color $setup(%s)",
-	      wn+1, check_perm [id].selected ? "marked":"black");
-#endif
 }
 
 #else
@@ -245,7 +236,6 @@ static void init_chmod (void)
 #define XTRACT(i) BY+chmod_but[i].y-single_set, BX+chmod_but[i].x, \
      chmod_but[i].ret_cmd, chmod_but[i].flags, _(chmod_but[i].text), 0, 0, NULL
 
-    tk_new_frame (ch_dlg, "b.");
     for (i = 0; i < BUTTONS; i++) {
 	if (i == 2 && single_set)
 	    break;
@@ -255,7 +245,6 @@ static void init_chmod (void)
 
 
 #define XTRACT2(i) 0, _(check_perm [i].text), NULL
-    tk_new_frame (ch_dlg, "c.");
     for (i = 0; i < PERMISSIONS; i++) {
 	check_perm[i].check = check_new (PY + (PERMISSIONS - i), PX + 2,
 					 XTRACT2 (i));
@@ -324,6 +313,9 @@ void chmod_cmd (void)
     int i;
     struct stat sf_stat;
 
+#if 0
+    /* Don't do things like this: you do not want to enumerate all
+       filesystems that can not support chmod, here. */
     if (!vfs_current_is_local ()) {
 	if (vfs_current_is_extfs ()) {
 	    message (1, _(" Oops... "),
@@ -335,6 +327,7 @@ void chmod_cmd (void)
 	    return;
 	}
     }
+#endif
 
     do {			/* do while any files remaining */
 	init_chmod ();
@@ -357,7 +350,6 @@ void chmod_cmd (void)
 	    check_perm[i].selected = 0;
 	}
 
-	tk_new_frame (ch_dlg, "l.");
 	/* Set the labels */
 	c_fname = name_trunc (fname, 21);
 	add_widget (ch_dlg, label_new (FY+2, FX+2, c_fname, NULL));
@@ -368,7 +360,6 @@ void chmod_cmd (void)
 	sprintf (buffer, "%o", c_stat);
 	statl = label_new (FY+4, FX+2, buffer, NULL);
 	add_widget (ch_dlg, statl);
-	tk_end_frame ();
 	
 	run_dlg (ch_dlg);	/* retrieve an action */
 	
