@@ -58,6 +58,7 @@
 #include "view.h"		/* view() */
 #include "dialog.h"		/* query_dialog, message */
 #include "file.h"		/* the file operations */
+#include "fileopctx.h"
 #include "find.h"		/* do_find */
 #include "hotlist.h"
 #include "tree.h"
@@ -325,7 +326,8 @@ void do_edit_at_line (const char *what, int start_line)
 #ifdef USE_INTERNAL_EDIT
     if (use_internal_edit){
 	edit (what, start_line);
-	reread_cmd ();
+	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+	repaint_screen ();
 	return;
     }
 #endif
@@ -335,7 +337,8 @@ void do_edit_at_line (const char *what, int start_line)
 	    editor = get_default_editor ();
     }
     execute_internal (editor, what);
-    reread_cmd ();
+    update_panels (UP_OPTIMIZE, UP_KEEPSEL);
+    repaint_screen ();
 }
 
 void
@@ -359,7 +362,7 @@ void edit_cmd_new (WPanel *panel)
 void copy_cmd (void)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, OP_COPY, NULL)){
+    if (panel_operate (cpanel, OP_COPY, NULL, TRUE)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -368,7 +371,7 @@ void copy_cmd (void)
 void ren_cmd (void)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, OP_MOVE, NULL)){
+    if (panel_operate (cpanel, OP_MOVE, NULL, TRUE)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -377,7 +380,7 @@ void ren_cmd (void)
 void copymove_cmd_with_default (int copy, char *thedefault)
 {
     save_cwds_stat ();
-    if (panel_operate (cpanel, copy ? OP_COPY : OP_MOVE, thedefault)){
+    if (panel_operate (cpanel, copy ? OP_COPY : OP_MOVE, thedefault, TRUE)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -385,23 +388,30 @@ void copymove_cmd_with_default (int copy, char *thedefault)
 
 void mkdir_cmd (WPanel *panel)
 {
+    char *tempdir;
     char *dir;
     
     panel = get_a_panel (panel);
-    dir = input_expand_dialog (_(" Mkdir "), _(" Enter directory name:") , "");
+    dir = input_expand_dialog (_("Create a new Directory"), _(" Enter directory name:") , "");
     
     if (!dir)
 	return;
-
+    
+    if (dir [0] && (dir [0] == '/' || dir [0] == '~'))
+	    tempdir = strdup (dir);
+    else
+	    tempdir = concat_dir_and_file (panel->cwd, dir);
+    free (dir);
+    
     save_cwds_stat ();
-    if (my_mkdir (dir, 0777) == 0){
-	update_panels (UP_OPTIMIZE, dir);
+    if (my_mkdir (tempdir, 0777) == 0){
+	update_panels (UP_OPTIMIZE, tempdir);
 	repaint_screen ();
 	select_item (cpanel);
-	free (dir);
+	free (tempdir);
 	return;
     }
-    free (dir);
+    free (tempdir);
     message (1, MSG_ERROR, "  %s  ", unix_error_string (errno));
 }
 
@@ -409,7 +419,7 @@ void delete_cmd (void)
 {
     save_cwds_stat ();
 
-    if (panel_operate (cpanel, OP_DELETE, NULL)){
+    if (panel_operate (cpanel, OP_DELETE, NULL, TRUE)){
 	update_panels (UP_OPTIMIZE, UP_KEEPSEL);
 	repaint_screen ();
     }
@@ -612,15 +622,16 @@ static int check_for_default(char *default_file, char *file)
     struct stat s;
     long   count = 0;
     double bytes = 0;
+    FileOpContext *ctx;
     
     if (mc_stat (file, &s)){
 	if (mc_stat (default_file, &s)){
 	    return -1;
 	}
-	create_op_win (OP_COPY, 0);
-        file_mask_defaults ();
-	copy_file_file (default_file, file, 1, &count, &bytes, 1);
-	destroy_op_win ();
+	ctx = file_op_context_new ();
+	file_op_context_create_ui (ctx, OP_COPY, 0);
+	copy_file_file (ctx, default_file, file, 1, &count, &bytes, 1);
+	file_op_context_destroy (ctx);
     }
     return 0;
 }
