@@ -25,7 +25,7 @@
 #include "color.h"
 #include "x.h"
 
-/* "$Id: color.c,v 1.11 1998/12/29 16:52:56 norbert Exp $" */
+/* "$Id: color.c,v 1.12 1999/01/10 08:55:50 unammx Exp $" */
 
 /* To avoid excessive calls to ncurses' has_colors () */
 int   hascolors = 0;
@@ -294,7 +294,7 @@ void init_colors (void)
 	}
 #endif
 
-#if defined HAVE_SLANG && !defined(OS2_NT)
+#if defined HAVE_SLANG && !defined(HAS_DIRECT_COLOR_ACCESS)
 	if (use_colors) { /* Hack to make COLOR_PAIR(DEFAULT_COLOR_INDEX) 
                              be the default fg/bg of the terminal */
 	    char *Norm_Vid = SLtt_tgetstr ("me");
