@@ -33,9 +33,9 @@
 #include "main.h"
 #include "key.h"		/* For mi_getch() */
 #include "dlg.h"		/* draw_box, yes I know, it's silly */
-#include "background.h"		/* we_are_background definition */
+#include "fileopctx.h"
 
-/* "$Id: dialog.c,v 1.4 1999/01/01 02:05:05 unammx Exp $" */
+/* "$Id: dialog.c,v 1.5 1999/01/14 01:11:09 unammx Exp $" */
 
 Refresh *refresh_list = 0;
 
