@@ -141,7 +141,7 @@
 #include <glib.h>
 #include "popt.h"
 
-/* "$Id: main.c,v 1.71 1999/01/12 09:08:42 unammx Exp $" */
+/* "$Id: main.c,v 1.72 1999/01/14 01:11:20 unammx Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -2811,7 +2811,7 @@ compatibility_move_mc_files (void)
 #endif
 
 void
-mc_tree_store_load ()
+mc_tree_store_load (void)
 {
 	char *tree_file;
 	
@@ -2822,7 +2822,7 @@ mc_tree_store_load ()
 }
 
 void
-mc_tree_store_save ()
+mc_tree_store_save (void)
 {
 	char *tree_file;
 
