@@ -29,6 +29,8 @@
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>	/* For chdir(), readlink() and getwd()/getcwd() */
 #endif
+/*#include "mc.h"*/
+
 #include "mem.h"
 #include "mad.h"
 #include "global.h"
@@ -53,7 +55,6 @@
 #include "../vfs/vfs.h"
 #include "../vfs/extfs.h"
 #include <glib.h>
-
 #ifdef NEEDS_DRIVE_H
 # include "drive.h"
 #endif
@@ -276,18 +277,6 @@ string_file_nlinks (file_entry *fe, int len)
     return buffer;
 }
 
-char *
-string_file_owner (file_entry *fe, int len)
-{
-    return get_owner (fe->buf.st_uid);
-}
-
-char *
-string_file_group (file_entry *fe, int len)
-{
-    return get_group (fe->buf.st_gid);
-}
-
 char *
 string_file_size (file_entry *fe, int len)
 {
@@ -335,12 +324,42 @@ string_file_ctime (file_entry *fe, int len)
 
 #ifdef HAVE_GNOME
 /* In GNOME, the CList truncates the names */
+char *
+string_file_owner (file_entry *fe, int len)
+{
+    return get_owner (fe->buf.st_uid);
+}
+
+char *
+string_file_group (file_entry *fe, int len)
+{
+    return get_group (fe->buf.st_gid);
+}
+
 char *
 string_file_name (file_entry *fe, int len)
 {
 	return fe->fname;
 }
 #else
+char *
+string_file_owner (file_entry *fe, int len)
+{
+    if (len)
+	return name_trunc (get_owner (fe->buf.st_uid), len);
+    else
+	return get_owner (fe->buf.st_uid);
+}
+
+char *
+string_file_group (file_entry *fe, int len)
+{
+    if (len)
+	return name_trunc (get_group (fe->buf.st_gid), len);
+    else
+	return get_group (fe->buf.st_gid);
+}
+
 char *
 string_file_name (file_entry *fe, int len)
 {
@@ -1655,7 +1674,9 @@ static void
 move_selection (WPanel *panel, int lines)
 {
     int new_pos;
+#ifndef HAVE_X
     int adjust = 0;
+#endif
 
     new_pos = panel->selected + lines;
     if (new_pos >= panel->count)
@@ -2543,7 +2564,8 @@ panel_re_sort (WPanel *panel)
     char *filename;
     int  i;
 
-    g_return_if_fail (panel != NULL);
+    if (panel == NULL)
+	    return;
     
     filename = strdup (selection (panel)->fname);
     unselect_item (panel);
