@@ -66,7 +66,7 @@ extern int ftpfs_use_passive_connections;
 extern int ftpfs_use_unix_list_options;
 #endif
 
-/* "$Id: setup.c,v 1.18 1998/12/21 23:31:06 norbert Exp $" */
+/* "$Id: setup.c,v 1.19 1999/01/17 04:17:23 unammx Exp $" */
 
 #ifdef USE_VFS
 extern int vfs_timeout;
@@ -244,9 +244,11 @@ static struct {
     { "editor_syntax_highlighting", &option_syntax_highlighting },
 #endif
 
-#ifndef HAVE_GNOME
+#ifdef HAVE_GNOME
+    { "tree_panel_visible", &tree_panel_visible },
+#else
     { "nice_rotating_dash", &nice_rotating_dash },
-    { "horizontal_split", &horizontal_split },
+    { "horizontal_split",   &horizontal_split },
 #endif
     { 0, 0 }
 };
