@@ -205,7 +205,7 @@ tree_store_load (char *name)
 	int len, common;
 	int do_load;
 	
-	g_return_if_fail (name != NULL);
+	g_return_val_if_fail (name != NULL, TRUE);
 
 	if (ts.loaded)
 		return TRUE;
@@ -274,6 +274,7 @@ tree_store_load (char *name)
 					e->scanned = scanned;
 					strcpy (oldname, name);
 				}
+			free (name);
 		}
 		fclose (file);
 	}
@@ -659,6 +660,7 @@ tree_store_end_check (void)
 	}
 
 	free (ts.check_name);
+	ts.check_name = NULL;
 }
 
 tree_entry *
@@ -679,8 +681,8 @@ tree_store_rescan (char *dir)
 		for (dp = mc_readdir (dirp); dp; dp = mc_readdir (dirp)){
 			char *full_name;
 
-			if (dp->d_name [0] == '.' &&
-			    dp->d_name [1] == 0 || (dp->d_name [1] == '.' && dp->d_name [2] == 0))
+			if ((dp->d_name [0] == '.' && dp->d_name [1] == 0)
+			    || (dp->d_name [1] == '.' && dp->d_name [2] == 0))
 				continue;
 
 			full_name = concat_dir_and_file (dir, dp->d_name);
@@ -733,6 +735,9 @@ tree_store_opendir (char *path)
 		if (!entry)
 			return NULL;
 	}
+
+	if (entry->next == NULL)
+		return NULL;
 	
 	scan = xmalloc (sizeof (tree_scan), "");
 	scan->base = entry;
