@@ -125,7 +125,7 @@ int altered_magic_flag = 0;
 int altered_nroff_flag = 0;
 /* }}} */
 
-/* "$Id: view.c,v 1.23 1999/01/10 08:56:10 unammx Exp $" */
+/* "$Id: view.c,v 1.24 1999/01/14 01:04:53 unammx Exp $" */
 
 static char hex_char[] = "0123456789ABCDEF";
 
@@ -1472,7 +1472,7 @@ search (WView *view, char *text, int (*search)(WView *, char *, char *, int))
 
 	if ((count++ % 32) == 0)
 	    x_flush_events ();
-	if (!d->running)
+	if (verbose && !d->running)
 	    break;
 #endif
 	if (p >= update_activate){
