@@ -20,7 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>	/* For free() */
 #include <string.h>
-#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(OS2_NT)
+#if (!defined(__IBMC__) && !defined(__IBMCPP__)) && !defined(HAS_NO_TERMIOS_H)
 #    include <termios.h>
 #endif
 #include "mad.h"
@@ -35,7 +35,7 @@
 #include "layout.h"
 #include "global.h"
 
-/* "$Id: win.c,v 1.3 1998/12/02 05:18:54 unammx Exp $" */
+/* "$Id: win.c,v 1.4 1999/01/10 08:56:12 unammx Exp $" */
 
 typedef void (*fnptr)(void);
 
